#!/usr/bin/perl -w

# RS232 testing
# to see what happens eg. when using crappy prolific pl2303 (windows drivers)
# and other cases

if (@ARGV < 1) {
    usage();
}

sub usage
{
    print "usage:\n";
    print "\tperl bin/rs232test.pl [speedword] [:port]\n";
	print "\n[speedword]\n"; print "\t19200, 38400, 57600, 115200\n\n";
	print "[:port]\n";
	print "\t:COM1, :/dev/ttyS0\n\n";
	print "Start one process on both ends of the line. Possibly wired one way RX=>TX\n";
	print "Loopback might work, but good test is using 2 PCs\n";
	print "Press <CTRL-C> to exit\n";

    undef $ob if defined $ob;
    exit;
}

use vars qw( $OS_win $ob $port );

#$maxerr = 59;
$sendchunksize = 80; # send 10..100 bytes at a time
$bootl_speed = 19200;
$timeout = 200;
$last_sent_char = ord( '0' ); # sending '0' first (than count up)

BEGIN {
$OS_win = ($^O eq "MSWin32") ? 1 : 0;
if ($OS_win) {
  eval "use Win32::SerialPort";
  die "$@\n" if ($@);
} else {
  eval "use Device::SerialPort";
  die "$@\n" if ($@);
}
} # BEGIN


# Install like this: perl -MCPAN -e 'install Device::SerialPort'
# If you are running Windows 95 or later, you want the Win32::SerialPort
# module instead of this one. It has a compatible user interface.

process_param();

#print STDERR "using serialport:$port\n";
if($OS_win){
	$port = 'COM1' if !defined $port;
	$ob = Win32::SerialPort->new ($port);
} else {
	$port = '/dev/ttyS0' if !defined $port;
	$ob = Device::SerialPort->new ($port);
}
################################################################


serial_init();

###############################################################

sub serial_init
{
# found on http://members.aol.com/Bbirthisel/alpha.html
# should work on multiple platforms

die "Can't open serial port $port: $^E\n" unless ($ob);

# save the state of the comm port for later
$old_baud = $ob->baudrate;
$old_parity = $ob->parity;
$old_data = $ob->databits;
$old_stop = $ob->stopbits;
$old_hshake = $ob->handshake;

# try using saved configuration

$ob->baudrate($bootl_speed);
$ob->parity('none');
$ob->databits(8);
$ob->stopbits(1);
$ob->handshake('none');
$ob->binary(T);

# without this communication returned too few bytes sometimes
$ob->read_interval(100) if ($OS_win);

$ob->write_settings || undef $ob;

$tmout = $timeout;
# ok, now port is configured
}

##############################################################################

$ob->baudrate($bootl_speed);
$ob->lookclear();
$ob->read_const_time(500);

# infinite LOOP:
while(1){
	comm_send_chunk();
	$b = srl_wait( $sendchunksize ); # $sendchunksize-1 ?
	processbuffer( $b );
}

$ob->purge_rx();

###############################################x

# restore state of comm port
$ob->baudrate($old_baud);
$ob->parity($old_parity);
$ob->databits($old_data);
$ob->stopbits($old_stop);
$ob->handshake($old_hshake);

undef $ob if defined $ob;

######################################################

# send last_sent_char++, sendchunksize bytes at a time
# saving result to last_sent_char
sub comm_send_chunk
{
	my $cnt;
	my $count_out = 0;
	
	for ($cnt = 0; $cnt < $sendchunksize; $cnt++ ){
    		$count_out += $ob->write( chr( $last_sent_char++ ) );
	}
    	warn "write failed\n" unless ($count_out);
    	warn "write incomplete\n" if ($count_out != $sendchunksize);
    	return $count_out;
}

# ----
sub processbuffer
{
	my $b = shift;
	my $recnow;
	my $len = length($b);
	@array= split(//, $b);
	my $g;
	for($g=0; $g < $len; $g++) {
		$recnow = ord($array[$g]);
		if( $recnow != $LAST_RECEIVED_BYTE + 1){
			print 'z'; # z means error
			# TODO: should we print \n sometimes ? Is windows capable of displaying characters ?
		}
		$LAST_RECEIVED_BYTE = $recnow;
	} # for

	print " $len\n";
}
#################################
sub process_param
{

#print "number of args:$#ARGV\n";
	$numbuf = 16; # default 
	$mde_exit = "mde00";	# turn off the wbo2
my $w;
	# number of args was checked earlier
   	while($w = shift @ARGV) {
 	if( $w =~ /^\d+$/){
                $bootl_speed = $w;
                print "serial bootloader speed set to $w\n";
                next;
        } elsif ($w =~ /^:(\S+)$/){
		$port = $1;
	} elsif ($w =~ /^\_(\d+)$/){
		$numbuf = $1;
	} elsif ($w =~ /^mde[a-fA-F\d]{2}$/){
		$mde_exit = $w;
	}
	} # while
}

#######################
sub comm_send
{
    my $str = shift ;
    $count_out = $ob->write($str);
    warn "write failed\n" unless ($count_out);
    warn "write incomplete\n" if ($count_out != length($str));
    return $count_out;
}

#######################
sub comm_recv
{
    my $b = shift;
    ($count_in, $string_in) = $ob->read($b);
    warn "read unsuccessful\n" unless ($count_in == $b);
    return ($count_in, $string_in);
}

sub flush
{
	my $b = shift;
	$ob->read_const_time($tmout);
	($count_in, $string_in) = $ob->read($b);
	return ($count_in, $string_in);
}


#################################################
# what's the reply???
sub srl_wait
{
	my $realcharcnt = shift;
	if (defined($realcharcnt)) {
	    $charcnt = $realcharcnt;
	} else {
	    $charcnt = 1;
	}
# TODO: if no reply comes within 2 sec, we should start saying 'v' -s or sg.
# finally report the problem (or let the user ctrl-c)
	# $ob->read($in_);
	$ob->read_const_time($tmout);
	my ($cnt, $in_) = comm_recv($charcnt);
	#sysread (SRL, $in_, 1, 0) or die "cannot readback from SRL $!";
	
	if(defined $in_){
		#print "got back $cnt:$in_ :".ord(substr($in_, 0, 1));
		#print ' '.ord(substr($in_, 1, 1)) if(length($in_) > 1);
		#print "\n";
	} else {
		print "nothing\n";
	}
	
	return $in_ if defined($realcharcnt) && ($realcharcnt > 0);
	return undef if $in_ eq "\r";	# success 
	return 1;					# error
}
